// SPDX-License-Identifier: MIT

use paste::paste;
use serde::{Deserialize, Serialize};
use std::fmt;

// get permission from type data, see CheckAuthRestApi comment
pub trait ToRestApiPermission {
    fn to_permission() -> RestApiPermission;
}

// Because we use permissions in type templates we define permissions twice through a macro:
// - RestApiPermission is an enum that containers all permissions e.g.
//   RestApiPermission::Admin
// - an extra RestApiPermissionXYZ type (e.g. RestApiPermissionAdmin) to be used
//   with CheckAuthRestApi
//
// The macro is a way of not repeating ourselves too much.
// We need paste! to creates tokens for RestApiPermissionXYZ ( [< and >] blocks)
macro_rules!define_permissions {
    ($($perm:ident),+) => {
        paste! {
            #[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, Deserialize, Serialize)]
            pub enum RestApiPermission {
                $( $perm, )*
            }
            impl RestApiPermission {
                pub fn all() -> &'static [RestApiPermission] {
                    // No easy way to get the static slice size:
                    // https://github.com/rust-lang/lang-team/issues/28
                    const SIZE: usize = 0usize $( + { stringify!($perm); 1 } )*;
                    static PERMISSIONS: [RestApiPermission; SIZE] = [
                        $( RestApiPermission::$perm, )*
                    ];
                    &PERMISSIONS
                }
            }
            $(
                pub enum [<RestApiPermission $perm>] {}

                impl ToRestApiPermission for [<RestApiPermission $perm>] {
                    fn to_permission() -> RestApiPermission {
                        RestApiPermission::$perm
                    }
                }
            )*
        }
    }
}

define_permissions!(
    Admin,
    ContainerAdmin,
    ContainerView,
    NetworkAdmin,
    NetworkView,
    Poweroff,
    Reboot,
    SwuInstall,
    SwuView
);

impl fmt::Display for RestApiPermission {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        fmt::Debug::fmt(self, f)
    }
}
